package dev.svitan.services

import kotlinx.serialization.Serializable
import org.jetbrains.exposed.sql.*
import org.jetbrains.exposed.sql.SqlExpressionBuilder.eq
import org.jetbrains.exposed.sql.transactions.transaction
import java.security.MessageDigest
import java.time.LocalDateTime
import java.time.format.DateTimeFormatter
import java.util.UUID

@Serializable
class AuthDTO(
    val id: String,
    val name: String,
    val createdAt: String
)

@Serializable
class NewAuthDTO(
    val name: String,
    val pin: String,
    val key: String
)

class AuthService {
    object Auths : Table("auths") {
        val id = uuid("id").autoGenerate()
        val name = text("name").uniqueIndex()
        val pin = text("pin")
        val key = text("key")
        val createdAt = text("created_at")

        override val primaryKey = PrimaryKey(id)
    }

    companion object {
        fun init() {
            transaction {
                SchemaUtils.create(Auths)
            }
        }

        fun create(action: NewAuthDTO): UUID = transaction {
            val now = LocalDateTime.now().format(DateTimeFormatter.ISO_LOCAL_DATE_TIME)
            Auths.insert {
                it[name] = action.name
                it[pin] = sha256(action.pin)
                it[key] = sha256(action.key)
                it[createdAt] = now
            }[Auths.id]
        }

        fun read(id: UUID): AuthDTO? = transaction {
            Auths.selectAll()
                .where { Auths.id eq id }
                .map {
                    AuthDTO(
                        it[Auths.id].toString(),
                        it[Auths.name],
                        it[Auths.createdAt],
                    )
                }.singleOrNull()
        }

        fun readAll(): List<AuthDTO> = transaction {
            Auths.selectAll().map {
                AuthDTO(
                    it[Auths.id].toString(),
                    it[Auths.name],
                    it[Auths.createdAt],
                )
            }
        }

        fun validate(id: UUID, pin: String, key: String): AuthDTO? = transaction {
            Auths.selectAll()
                .where { (Auths.id eq id) and (Auths.pin eq sha256(pin)) and (Auths.key eq sha256(key)) }
                .map {
                    AuthDTO(
                        it[Auths.id].toString(),
                        it[Auths.name],
                        it[Auths.createdAt]
                    )
                }
                .singleOrNull()
        }

        fun delete(id: UUID) {
            transaction {
                Auths.deleteWhere { Auths.id eq id }
            }
        }
    }
}

fun sha256(it: String): String {
    val bytes = it.toByteArray()
    val md = MessageDigest.getInstance("SHA-256")
    val digest = md.digest(bytes)
    return digest.joinToString("") { "%02x".format(it) }
}
